#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2016-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end


require 'optparse'
require 'optparse/time'
require 'ostruct'
require_relative 'fsobjects'
require_relative 'userio'

class ArgParser 
	# Returns a structure describing the options.
	#
	def self.parse(args)
		# The options specified on the command line will be collected in <b>options</b>.
		# Set default values here, before the call to OptionParser.new.
		options = OpenStruct.new
			
		op = OptionParser.new do |opts|
			
			opts.banner = "\nUsage" << ":\t%s <options>" %($0) 
			opts.banner << "\n  or" << ":\t%s -s [source] -t [target] -b" %($0)

			opts.separator ""
			opts.separator "Specific options:"
			opts.on('-c' << "DIRECTORY", '--' << "conf DIRECTORY", "Manipulate files in the named directory (Default: #{$DEF_CONF_DIR})") do |dir|
				options.conf_dir = dir
			end
			opts.on('-b', '--buttons', "Copy UserButton definitions") do 
				options.section = :buttons
			end

			opts.on('-s' << " SOURCE-FILE", '--source' << " SOURCE-FILE", "The configuration-file which contains the original sections (or a short name)") do |source|
				options.source = source
				options.action = :copy
			end

			opts.on('-t' << " TARGET-FILE", '--target' << " TARGET-FILE", "The configuration-file to which the copied sections shall be appended (or a short name)") do |target|
				options.target = target
				options.action = :copy
			end

			opts.on('-l', '--list', "list the available configuration files") do 
				options.action = :list_conf_files
			end

			opts.on('-n', '--names', "list the short names for the available programs") do 
				options.action = :list_short_names
			end

			opts.on('-a', '--all', "list available config-files, names and program versions") do 
				options.action = :list_all
			end

			opts.separator ""
			opts.separator ("Common options") << ':'

			
			# No argument, shows at tail.  This will print an options summary.
			opts.on_tail(("-h"), ("--help"), ("Show this message") ) do
				puts opts
				puts
				exit true
			end

			opts.on_tail(("-v"), ("--version"), ("Show version and program information") ) do
				UserIO::msg_version
				exit true
			end
		end

		begin
			op.parse!(args)
		rescue OptionParser::MissingArgument => err
			UserIO::msg_option_parse_error(err)
			exit false
		rescue OptionParser::InvalidOption => err
			UserIO::msg_option_parse_error(err)
			exit false
		end
		options
	end  # parse()
end  

