#!/usr/bin/ruby
#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2016-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
# require 'rubygems'
require 'escape'
require_relative 'argparser'
require_relative 'userio'
require_relative 'fsobjects'

# Currently unused
# $PROG_DIR = File::expand_path(File::dirname(__FILE__) ) << File::Separator if !defined?($PROG_DIR)
# $ICON_DIR = $PROG_DIR.dup << 'icon' << File::Separator if !defined?($ICON_DIR)

# prepare for future uses/avoid bitter tears
$WIN = (RUBY_PLATFORM =~ /mswin/)

# UNUSED
# $CONFIG = ENV['HOME'].dup << File::Separator << '.cpsobuttons.conf'

# $DEF_CONF_DIR = ENV['HOME'].dup << File::Separator << '.softmaker'
$DEF_CONF_DIR = ENV['HOME'].dup << File::Separator << 'SoftMaker/Settings'

# The main application-class.
class CpSoButtons

	def initialize(args)
		# remove this, if you want
		win_bail_out
		$APP_NAME = self.class.name
		options = ArgParser.parse(args)

		if(!options.conf_dir)
			guess_conf_dir(options)
		end
		if(options.action)
			self.send(options.action, options) 
		else
			UserIO::msg_missing_option
		end
	end

	def method_missing(method, *args)
		UserIO::msg_method_undefined(self.class.name, method.to_s)
		exit
	end

	private
	# If not given, try to read all information from files in the 
	# directory $DEF_CONF_DIR
	def guess_conf_dir(options)
		if(Dir.exists?($DEF_CONF_DIR) )
			options.conf_dir = $DEF_CONF_DIR
			UserIO::msg_info_conf_dir(options.conf_dir)
		else
			UserIO::msg_no_config("Where are those files? Use option '-c'")
		end
	end

	# This method copies sections from the source- to the target-file.
	# short names (e.g. tmfo16 -> pmfo16) and full ini-paths should be
	# okay.
	def copy(options)
		src = options.source
		trg = options.target
		section = options.section
		if(section)
			if(src && trg)
				if(src != trg)
					# Currently, only buttons can be copied.
					if(:buttons == section)
						FSObjects.copy_buttons(src, trg, section, options.conf_dir)
					else
						# TODO
						# FSObjects.copy(src, trg, section)
					end
				else
					UserIO::msg_cannot_copy('Source- and target-file are the same.')
				end
			else
				UserIO::msg_cannot_copy((src ? 'Target' : 'Source') << " not specified.")
			end
		else
			UserIO::msg_no_section_given('Copy what? (hint: copy buttons with -b)')
		end
	end
	# lists all program-names and configuration-files.
	def list_all(options)
		UserIO::msg_available_information
		list = FSObjects.complete(options.conf_dir)
		puts list
		puts
	end
	# lists the short application designators which correspond
	# to the existing configuration-files
	def list_short_names(options)
		UserIO::msg_short_progs
		list = FSObjects.short_names(options.conf_dir).sort
		puts list
		puts
	end
	# lists all existing configuration-files
	def list_conf_files(options)
		UserIO::msg_list_configs
		list = FSObjects.read_config(options.conf_dir).sort.join("\n")
		puts list
		puts
	end
	# Exits the program if it is run on Windows.
	def win_bail_out
		if $WIN
			UserIO.msg_win_bailout
			exit false
		end
	end
end

# ---- TEST ----

if __FILE__ == $0
	cpbt = CpSoButtons.new(ARGV)
end

# EOF
